import sys, os, shutil

from const import *

# WineDoors global instances
from preferences import preferences
from packlist import packlist
from utils import Download
from wine import wine
from log import log

# Parser
from xml.sax import make_parser
from xml.sax.handler import ContentHandler

class ApplicationPackParse( ContentHandler ):
	def __init__( self, pack_data ):
		self.in_app_pack = False
		ContentHandler.__init__( self )
		self.pack_data = pack_data
		self.requires = {}
		self.data = []
		
	def startElement( self, name, attrs ):
		if name == "ApplicationPack":
			self.in_app_pack = True
		elif name == "crack" and self.in_app_pack:
			self.crack_required = attrs.get( 'required' ) == "1"
		elif name == "requires" and self.in_app_pack:
			self.requires[attrs.get( 'name' )] = attrs.get( 'version' )
			
		
	def endElement( self, name ):
		if name == "ApplicationPack":
			self.in_app_pack = False
			self.pack_data['requires'] = self.requires
		elif name == "crack" and self.in_app_pack:
			self.pack_data["crack_required"] = self.crack_required
			text = ''.join( self.data )
			self.pack_data[name] = text
		elif self.in_app_pack:
			text = ''.join( self.data )
			self.pack_data[name] = text
		self.data = []

	def characters( self, string ):
		self.data.append( string.rstrip() )
			
	def error( self, exception ):
		log.Error( "%s\n" % exception )

class ApplicationPack:
	def __init__( self, operation, pack, version=None, repo=None ):
		'''
		pack can be a remote file, local file or pack short name.
		pack short names require repo to be set. If no version is specified
		default version will be installed, if no default version is specified
		all versions will be installed.
		'''
		self.app_pack = []
		self.pack = []
		if pack.startswith ( "http://" ):
			self.type = REMOTE
		elif os.path.isfile( pack ):
			self.type = LOCAL
		elif repo:
			packlist.Update(repo)
			self.pack = packlist.pack[pack]
			self.repo = repo
			self.repo_uri = packlist.GetRepoURI( repo )
			self.type = REPO
		elif pack in packlist.available:
			self.pack = packlist.available[pack]
			repo_data = self.app_pack['repo']
			repo = ''.join( repo_data[0:1] )
			repo_uri = ''.join( repo_data[1:] )
			self.type = PACKLIST
		else:
			log.Error( "Can not determine the application pack type" )
			return

		if not version:
			self.version = ALL
		else:
			self.version = version

		if operation == DEPS:
			self.CheckRequires()
		elif operation == INSTALL:
			self.InstallApplication()
		elif operation == DRYRUN:
			self.DryInstall()
		
	def Parse( self ):
		app_pack_xml = self.pack_path + "pack.xml"
		handler = ApplicationPackParse( self.app_pack )
		parser = make_parser()
		parser.setContentHandler( handler )
		try:
			parser.parse( open( file ) )
		except:
			parser.parse( file )

	def Download ( self, file ):
		if self.repo:
			remote_file = self.repo_uri + "/" + file
		else:
			remote_file = file
			( dir, file ) = os.path.split( remote_file )
		
		self.cache_path = preferences["wineroot"] + "/wine-doors/apppacks/"
		self.cache_file = self.cache_path + file
		( pack_folder, ext ) = os.path.splitext( file )
		self.pack_path = self.cache_path + pack_folder + "/"
		utils.Download ( remote_file, self.cache_file, True )
		
	def Extract ( self ):
		cwd = os.getcwd()
		os.chdir(self.cache_path)
		if self.cache_file.endswith( "bz2" ):
			os.system( "tar -xjf " + self.cache_file )
		elif self.cache_file.endswith( "gz" ):
			os.system( "tar -xzf " + self.cache_file )
		elif self.cache_file.endswith( "wdi" ):
			os.system( "tar -xzf " + self.cache_file )
		os.chdir(cwd)
		
	def CheckRequires( self ):
		# This should only load packs from the same repo
		pass
		
	def DryInstall( self ):
		pass
		
	def InstallApplication( self ):
		"""
		Install an application using an application pack
		"""
		
		if self.version == ALL:
			# easiest way to do this is to create new instances of application 
			# pack passing the version for each item in sequence
			# This will require some good use of logging in other instantiated
			# areas of the code
			pass # Looping each version doing the following
		else:
			pack_file = self.pack['versions'][version][4:] # Thats crack!
			self.Download(pack_file)
			self.Extract()
			self.Parse()
			wine.Execute( self.app_pack['installer'] )
			if wine.WaitForExit() == False:
				wine.Kill()
	
			# TODO: Add to installed.packlist.xml
			for script in self.scripts.values():
				self.InstallFile( script, SCRIPT )
			self.InstallFile( self.desktop, LAUNCHER )
			if self.regfile:
				self.InstallFile( self.regfile, REGISTRY )
			return False	
		
	def InstallFile( self, file, type ):
		if type == LAUNCHER:
			if os.environ['USER'] == "root":
				desktop_path = "/usr/share/applications/"
			else:
				desktop_path = os.environ['HOME']+"/.local/share/applications/"
			if not os.path.isdir( desktop_path ):
				os.makedirs( desktop_path )
			shutil.copy( file, desktop_path )
		elif type == SCRIPT:
			if not os.path.isdir( os.environ['HOME']+"/bin/" ):
				os.makedirs( os.environ['HOME']+"/bin/" )
			shutil.copy( file, os.environ['HOME']+"/bin/" )
		elif type == REGISTRY:
			wine.Execute( "regedit "+file )
## Unit testing 
def usage():
	print "Unit testing of pack.py"
	print "Usage: python pack.py <class> <file>"
	print "  <class> one of ApplicationPackParse, ApplicationPack"
	print "  <file> file to feed into the class as a unit"
	
if __name__ == "__main__":
	if len( sys.argv ) == 3:
		if sys.argv[1] == "ApplicationPack":
			print "Testing Class: ApplicationPack"
			print "  Parsing application pack from " + sys.argv[2]			
			apppack = ApplicationPack( "dryrun", sys.argv[2] )
		elif sys.argv[1] == "ApplicationPackParse":
			print "Testing Class: ApplicationPackParse"
			print "  Parsing application pack xml from " + sys.argv[2]
			pack = []
			handler = ApplicationPackParse( pack )
			parser = make_parser()
			parser.setContentHandler( handler )
			parser.parse( sys.argv[2] )
			printNestedStuff( pack )   
		else:
			print sys.argv[1] + " is not a valid class"
			usage()	   
	else:
		usage()